package johan;

import java.util.Set;

import com.ingenico.pclservice.PclService;
import com.ingenico.pclutilities.PclUtilities;
import com.ingenico.pclutilities.PclUtilities.BluetoothCompanion;
import anywheresoftware.b4a.BA.DependsOn;
import anywheresoftware.b4a.BA.Hide;
import anywheresoftware.b4a.BA.Permissions;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;
import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.ServiceConnection;
import android.content.SharedPreferences;
import android.os.Handler;
import android.os.IBinder;

import com.ingenico.pclutilities.SslObject;
import anywheresoftware.b4a.BA;
import posintegrado.ingenico.com.mposintegrado.mposLib;

@BA.ActivityObject
@ShortName("PCLServices")
@DependsOn(values = { "mposintegrado.aar" })
@Permissions(values = { "android.permission.WRITE_EXTERNAL_STORAGE", "android.permission.INTERNET",
        "android.permission.ACCESS_NETWORK_STATE", "android.permission.BLUETOOTH", "android.permission.BLUETOOTH_ADMIN",
        "android.permission.ACCESS_WIFI_STATE" })
@Version(1.00F)
public abstract class PCLServicio extends Activity {

    protected PclService mPclService = null;
    protected boolean mBound = false;
    protected PclUtilities mPclUtil;
    protected mposLib mposLibobj;
    protected boolean mServiceStarted;
    protected CharSequence mCurrentDevice;
    public Handler h;
    @Hide
    public static BA myba;

    public void Initialize(BA ba) {
        PCLServicio.myba = ba;
        this.h = mHandler();
    }

    private Handler mHandler() {
        return null;
    }

    // Implement ServiceConnection
    class PclServiceConnection implements ServiceConnection {
        public void onServiceConnected(ComponentName className, IBinder boundService) {
            /*
             * We've bound to LocalService, cast the IBinder and get
             * LocalService instance
             */
            PclService.LocalBinder binder = (PclService.LocalBinder) boundService;
            mPclService = (PclService) binder.getService();
            onPclServiceConnected();
        }

        public void onServiceDisconnected(ComponentName className) {
            mPclService = null;
        }
    };

    private PclServiceConnection mServiceConnection;

    // You can call this method in onCreate for instance
    protected void initService() {
        if (!mBound) {
            mServiceConnection = new PclServiceConnection();
            Intent intent = new Intent(this, PclService.class);
            mBound = bindService(intent, mServiceConnection,
                    Context.BIND_AUTO_CREATE);
        }
    }

    protected void releaseService() {
        if (mBound) {
            unbindService(mServiceConnection);
            mBound = false;
        }
    }

    private class StateReceiver extends BroadcastReceiver {

        private PCLServicio ViewOwner = null;

        @SuppressLint("UseValueOf")
        public void onReceive(Context context, Intent intent) {
            String state = intent.getStringExtra("state");
            ViewOwner.onStateChanged(state);
        }

        StateReceiver(PCLServicio receiver) {
            super();
            ViewOwner = receiver;
        }
    }

    private StateReceiver m_StateReceiver = null;

    private void initStateReceiver() {
        if (m_StateReceiver == null) {
            m_StateReceiver = new StateReceiver(this);
            IntentFilter intentfilter = new IntentFilter("com.ingenico.pclservice.intent.action.STATE_CHANGED");
            registerReceiver(m_StateReceiver, intentfilter);
        }
    }

    private void releaseStateReceiver() {
        if (m_StateReceiver != null) {
            unregisterReceiver(m_StateReceiver);
            m_StateReceiver = null;
        }
    }

    abstract void onStateChanged(String state);

    abstract void onPclServiceConnected();

    @Override
    protected void onResume() {
        super.onResume();
        initStateReceiver();
    }

    @Override
    protected void onPause() {
        super.onPause();
        releaseStateReceiver();
    }

    public boolean isCompanionConnected() {
        boolean bRet = false;
        if (mPclService != null) {
            byte result[] = new byte[1];
            {
                if (mPclService.serverStatus(result) == true) {
                    if (result[0] == 0x10) {
                        bRet = true;
                    }
                }
            }
        }
        return bRet;
    }

    public void startPclService() {
        if (!mServiceStarted) {
            SharedPreferences settings = getSharedPreferences("PCLSERVICE", MODE_PRIVATE);
            boolean enableLog = settings.getBoolean("ENABLE_LOG", true);
            Intent i = new Intent(this, PclService.class);
            i.putExtra("PACKAGE_NAME", "posintegrado.ingenico.com.sampleandroidapp");
            i.putExtra("FILE_NAME", "pairing_addr.txt");
            i.putExtra("ENABLE_LOG", enableLog);
            if (getApplicationContext().startService(i) != null)
                mServiceStarted = true;
        }
    }

    private void stopPclService() {
        if (mServiceStarted) {
            Intent i = new Intent(this, PclService.class);
            if (getApplicationContext().stopService(i))
                mServiceStarted = false;
        }
    }

    public void StartComunication() {
        /* Variable de control si ya se encontró un dispositivo */
        boolean bFound = false;
        /*
         * Se obtiene con esto la lista de los dispositivos ingenico paired con el
         * terminal
         */
        Set<BluetoothCompanion> btComps = mPclUtil.GetPairedCompanions();
        if (btComps != null && (btComps.size() > 0)) {
            /* Loop through paired devices */
            for (BluetoothCompanion comp : btComps) {
                /* Aca se revisa si el dispositivo esta activo y lo define como el actual */
                if (comp.isActivated()) {
                    bFound = true;
                    mCurrentDevice = comp.getBluetoothDevice().getAddress() + " - " +
                            comp.getBluetoothDevice().getName();
                } else {
                    /* Se activa el dispositivo */
                    mPclUtil.ActivateCompanion(comp.getBluetoothDevice().getAddress());
                    return;
                }
            }
        }
    }

    public void SaleExample() {
        mposLibobj = new mposLib(mPclService);
        String stx = "02";
        String ext = "03";

        String mensajeriaTrx = "0200|20000|123456|||0";
        /* Convierto mi string de trx a hex */
        String trxToHex = mposLibobj.convertStringToHex(mensajeriaTrx);
        /* Luego calculo el largo de mi trama en hex (LRC) */
        String obtenerLrc = calcularLRC(trxToHex);
        /* Ahora armo el comando completo de trx */
        String trxCompleta = stx + trxToHex + ext + obtenerLrc;
        /* Envio el comando completo para que el POS integrado Bluetooth lo procese */
        mposLibobj.stratTransaction(trxCompleta);
    }

    private String calcularLRC(String input) {
        String LRC = "";
        int uintVal = 0;
        if (input != "") {
            byte[] arrayofhex = hexStringToByteArray(input);
            for (int count = 1; count < arrayofhex.length; ++count) {
                if (count == 1) {
                    uintVal = arrayofhex[count - 1] ^ arrayofhex[count];
                } else {
                    uintVal ^= arrayofhex[count];
                }
            }
        }
        LRC = Integer.toHexString(uintVal).toUpperCase();
        int f = LRC.length();
        if (f == 2) {
            return LRC;
        } else {
            char[] chars = LRC.toCharArray();
            StringBuffer hex = new StringBuffer();
            for (int i = 0; i < chars.length; i++) {
                hex.append(Integer.toHexString(chars[i]));
            }
            return hex.toString();
        }
    }

    mposLibobj.setOnTransactionFinishedListener(new mposLib.onTransactionFinishedListener() {
        public void onFinish(String response) {
        /*EJEMPLO DE RESPONSE*/
            String respToString = mposLibobj.convertHexToString(response);
            //Log.i("Respuesta response: ", respToString);
            BA.Log("Respuesta response: " + respToString);
        }
    });

}
